local addonName, QuestKing = ...

-- options
local opt = QuestKing.options

-- Tracker

local Tracker = CreateFrame("Frame", nil, UIParent)
QuestKing.Tracker = Tracker

----

function Tracker:InitDrag()
	local dragOrigin = QuestKingDB.dragOrigin
	local dragX = QuestKingDB.dragX
	local dragY = QuestKingDB.dragY

	Tracker:ClearAllPoints()
	if (not dragX) or (not dragY) then
		Tracker:SetPoint(dragOrigin, UIParent, "CENTER", 0, 0)
	else
		Tracker:SetPoint(dragOrigin, UIParent, dragOrigin, dragX, dragY)
	end

	Tracker:CheckDrag()
end

local function trackerStartMoving (self)
	self.isMoving = true
	self:StartMoving()
end

local function trackerStopMoving (self)
	self.isMoving = false
	self:StopMovingOrSizing()

	local dragOrigin = QuestKingDB.dragOrigin
	local x, y
	if (dragOrigin == "TOPRIGHT") then
		x = self:GetRight() - UIParent:GetRight()
		y = self:GetTop() - UIParent:GetTop()
	elseif (dragOrigin == "BOTTOMRIGHT") then
		x = self:GetRight() - UIParent:GetRight()
		y = self:GetBottom()
	elseif (dragOrigin == "BOTTOMLEFT") then
		x = self:GetLeft()
		y = self:GetBottom()
	elseif (dragOrigin == "TOPLEFT") then
		x = self:GetLeft()
		y = self:GetTop() - UIParent:GetTop()
	end

	-- D(dragOrigin, x, y)
	self:ClearAllPoints()
	self:SetPoint(dragOrigin, UIParent, dragOrigin, x, y)

	QuestKingDB.dragX = x
	QuestKingDB.dragY = y
end

function Tracker:CheckDrag()
	local dragLocked = QuestKingDB.dragLocked

	if (dragLocked == false) then
		Tracker.titlebar:SetBackdropColor(0.6, 0, 0.6, 1)
		Tracker.titlebarText2:Show()

		Tracker:EnableMouse(true)
		Tracker:SetMovable(true)
		Tracker:RegisterForDrag("LeftButton")
		Tracker:SetScript("OnDragStart", Tracker.StartMoving)
		Tracker:SetScript("OnDragStop", trackerStopMoving)
	else
		if (Tracker.isMoving) then trackerStopMoving(Tracker) end

		Tracker.titlebar:SetBackdropColor(0, 0, 0, 0)
		Tracker.titlebarText2:Hide()

		Tracker:EnableMouse(false)
		Tracker:SetMovable(false)
		Tracker:RegisterForDrag()
		Tracker:SetScript("OnDragStart", nil)
		Tracker:SetScript("OnDragStop", nil)
	end
end

--

function QuestKing:CreateTracker ()
	-- size

	Tracker:SetWidth(opt.buttonWidth)
	Tracker:SetHeight(opt.titleHeight)

	-- titlebar

	local titlebar = CreateFrame("Button", nil, Tracker)
	titlebar:SetWidth(opt.buttonWidth)
	titlebar:SetHeight(opt.titleHeight)
	titlebar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, -100)
	titlebar:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	})
	titlebar:SetBackdropColor(0, 0, 0, 0)
	titlebar:EnableMouse(false)
	titlebar.parent = Tracker
	Tracker.titlebar = titlebar

	-- titlebar text

	local titlebarText = titlebar:CreateFontString(nil, opt.fontLayer)
	titlebarText:SetFont(opt.font, opt.fontSize, opt.fontStyle)
	titlebarText:SetJustifyH("RIGHT")
	titlebarText:SetTextColor(opt.colors.TrackerTitlebarText[1], opt.colors.TrackerTitlebarText[2], opt.colors.TrackerTitlebarText[3])
	titlebarText:SetShadowOffset(1, -1)
	titlebarText:SetShadowColor(0, 0, 0, 1)
	titlebarText:SetWordWrap(false)
	titlebarText:SetText("0/"..MAX_QUESTS)
	Tracker.titlebarText = titlebarText

	local titlebarText2 = titlebar:CreateFontString(nil, opt.fontLayer)
	titlebarText2:SetFont(opt.font, opt.fontSize, opt.fontStyle)
	titlebarText2:SetJustifyH("LEFT")
	titlebarText2:SetTextColor(0.7, 0.5, 0.9)
	titlebarText2:SetShadowOffset(1, -1)
	titlebarText2:SetShadowColor(0, 0, 0, 1)
	titlebarText2:SetWordWrap(false)
	titlebarText2:SetText("Unlocked - Drag me!")
	titlebarText2:SetPoint("TOPLEFT", titlebar, "TOPLEFT", 2, -1)
	Tracker.titlebarText2 = titlebarText2

	-- drag
	if (opt.allowDrag) then
		Tracker:InitDrag()
	else
		Tracker:SetPresetPosition()
	end	


	-- minimize button

	local minimizeButton = CreateFrame("Button", "QuestKing_TrackerMinimizeButton", titlebar)
	minimizeButton:SetWidth(15)
	minimizeButton:SetHeight(15)

	local tex = minimizeButton:CreateTexture()
	tex:SetPoint("CENTER")
	tex:SetTexture([[Interface\Buttons\UI-Quickslot2]])
	tex:SetWidth(22)
	tex:SetHeight(22)
	minimizeButton:SetNormalTexture(tex)

	local tex = minimizeButton:CreateTexture()
	tex:SetPoint("CENTER")
	tex:SetTexture([[Interface\Buttons\UI-Quickslot2]])
	tex:SetWidth(15)
	tex:SetHeight(15)
	minimizeButton:SetPushedTexture(tex)

	local tex = minimizeButton:CreateTexture()
	tex:SetPoint("CENTER")
	tex:SetTexture([[Interface\Buttons\UI-Quickslot-Depress]])
	tex:SetWidth(15)
	tex:SetHeight(15)
	minimizeButton:SetHighlightTexture(tex, "ADD")

	local label = minimizeButton:CreateFontString(nil, opt.fontLayer)
	label:SetFont(opt.font, opt.fontSize, opt.fontStyle)
	label:SetJustifyH("CENTER")
	label:SetJustifyV("MIDDLE")
	label:SetTextColor(opt.colors.TrackerTitlebarText[1], opt.colors.TrackerTitlebarText[2], opt.colors.TrackerTitlebarText[3])
	label:SetShadowOffset(1, -1)
	label:SetShadowColor(0, 0, 0, 1)
	label:SetPoint("CENTER", 1, 0.5)
	label:SetWordWrap(false)
	label:SetText("+")
	minimizeButton.label = label

	minimizeButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	minimizeButton:SetScript("OnClick", QuestKing.MinimizeButtonOnClick)
	
	if (opt.enableBackdrop) then
		Tracker:SetBackdrop(opt.backdropTable)
		Tracker:SetBackdropColor(unpack(opt.backdropColorDefault))

		minimizeButton:EnableMouseWheel()
		minimizeButton:SetScript("OnMouseWheel", QuestKing.MinimizeButtonOnMouseWheel)
	end

	-- mode button

	local modeButton = CreateFrame("Button", "QuestKing_TrackerModeButton", titlebar)
	modeButton:SetWidth(15)
	modeButton:SetHeight(15)

	local tex = modeButton:CreateTexture()
	tex:SetPoint("CENTER")
	tex:SetTexture([[Interface\Buttons\UI-Quickslot2]])
	tex:SetWidth(22)
	tex:SetHeight(22)
	modeButton:SetNormalTexture(tex)

	local tex = modeButton:CreateTexture()
	tex:SetPoint("CENTER")
	tex:SetTexture([[Interface\Buttons\UI-Quickslot2]])
	tex:SetWidth(15)
	tex:SetHeight(15)
	modeButton:SetPushedTexture(tex)

	local tex = modeButton:CreateTexture()
	tex:SetPoint("CENTER")
	tex:SetTexture([[Interface\Buttons\UI-Quickslot-Depress]])
	tex:SetWidth(15)
	tex:SetHeight(15)
	modeButton:SetHighlightTexture(tex, "ADD")

	local label = modeButton:CreateFontString(nil, opt.fontLayer)
	label:SetFont(opt.font, opt.fontSize, opt.fontStyle)
	label:SetJustifyH("CENTER")
	label:SetJustifyV("MIDDLE")
	label:SetTextColor(opt.colors.TrackerTitlebarText[1], opt.colors.TrackerTitlebarText[2], opt.colors.TrackerTitlebarText[3])
	label:SetShadowOffset(1, -1)
	label:SetShadowColor(0, 0, 0, 1)
	label:SetPoint("CENTER", 1, 0)
	label:SetWordWrap(false)
	label:SetText("Q")
	modeButton.label = label

	modeButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	modeButton:SetScript("OnClick", QuestKing.ModeButtonOnClick)

	-- layout

	minimizeButton:SetPoint("RIGHT", titlebar, "RIGHT", 0, 0)
	modeButton:SetPoint("RIGHT", minimizeButton, "LEFT", 1, 0)
	titlebarText:SetPoint("RIGHT", modeButton, "LEFT", -4, 0)

	titlebar:SetPoint("TOPLEFT", Tracker)


	if (QuestKingDBPerChar.dbAlpha) and (opt.dbAllowAlpha) then
		Tracker:SetAlpha(QuestKingDBPerChar.dbAlpha)
	else
		Tracker:SetAlpha(opt.trackerAlpha)
	end	
end

local measureFrame = nil
function Tracker:Resize (lastShown)
	if (not lastShown) then
		QuestKing.Tracker:SetHeight(opt.titleHeight)
	else
		if not measureFrame then
			measureFrame = CreateFrame("Frame", "QuestKing_MeasureFrame", UIParent)
			measureFrame:SetWidth(opt.buttonWidth)
			-- measureFrame:SetBackdrop({
			-- 	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			-- 	insets = { left = 0, right = 0, top = 0, bottom = 0 }
			-- })
			-- measureFrame:SetBackdropColor(0,1,0,0.5)
		end

		measureFrame:ClearAllPoints()
		measureFrame:SetPoint("TOPLEFT", QuestKing.Tracker)
		measureFrame:SetPoint("BOTTOMLEFT", lastShown)
		local height = measureFrame:GetHeight()
		QuestKing.Tracker:SetHeight(height)
	end
end


---- ###########################################
---- tracker related handlers
---- ###########################################


function Tracker:SetPresetPosition()
	local preset = opt.positionPresets[QuestKingDBPerChar.trackerPositionPreset]
	if not preset then
		QuestKingDBPerChar.trackerPositionPreset = 1
		preset = opt.positionPresets[1]
	end
	self:SetPoint(unpack(preset))
end

function Tracker:CyclePresetPosition()
	local new = QuestKingDBPerChar.trackerPositionPreset + 1
	if new > #opt.positionPresets then
		new = 1
	end
	QuestKingDBPerChar.trackerPositionPreset = new
	self:PositionTracker()
end

function QuestKing.MinimizeButtonOnClick (self, mouse, down)
	if (IsShiftKeyDown()) and (not opt.allowDrag) then
		Tracker:CyclePresetPosition()
	else
		if (mouse == "RightButton") then
			QuestKingDBPerChar.trackerCollapsed = 2
			PlaySound("igMiniMapClose")
		elseif (QuestKingDBPerChar.trackerCollapsed ~= 0) then
			QuestKingDBPerChar.trackerCollapsed = 0
			wipe(QuestKing.newlyAddedQuests)
			PlaySound("igMiniMapOpen")
		else
			QuestKingDBPerChar.trackerCollapsed = 1
			PlaySound("igMiniMapClose")
		end
		QuestKing:UpdateTracker()
	end
end

function QuestKing.MinimizeButtonOnMouseWheel (self, direction)
	local r, g, b, alpha = QuestKing.Tracker:GetBackdropColor()
	if (direction > 0) then
		alpha = alpha + opt.backdropAlphaStep
		if (alpha > 1) then alpha = 1 end

		QuestKing.Tracker:SetBackdropColor(r, g, b, alpha)
	elseif (direction < 0) then
		alpha = alpha - opt.backdropAlphaStep
		if (alpha < 0) then alpha = 0 end

		QuestKing.Tracker:SetBackdropColor(r, g, b, alpha)
	end
end

function QuestKing.ModeButtonOnClick (self, mouse, down)
	if mouse == "RightButton" then
		if IsAltKeyDown() then
			QuestKing:SetSuperTrackedQuestID(0)
		elseif QuestKingDBPerChar.displayMode ~= "combined" then
			QuestKingDBPerChar.displayMode = "combined"
		else
			QuestKingDBPerChar.displayMode = "quests"
		end
	else
		if QuestKingDBPerChar.displayMode == "quests" then
			QuestKingDBPerChar.displayMode = "achievements"
		else
			QuestKingDBPerChar.displayMode = "quests"
		end
	end
	QuestKing:UpdateTracker()
end


---- ###########################################
---- button-related handlers
---- ###########################################

function QuestKing:TitleButtonOnEnter (motion)
	local button = self.parent

	if ((button.mouseHandler) and (button.mouseHandler.TitleButtonOnEnter)) then
		button.mouseHandler.TitleButtonOnEnter(self, motion)
		return
	end
end

function QuestKing:TitleButtonOnLeave (motion)
	if (opt.tooltipScale) then
		local oldScale = GameTooltip.__QuestKingPreviousScale or 1
		GameTooltip.__QuestKingPreviousScale = nil
		GameTooltip:SetScale(oldScale)
	end
	GameTooltip:Hide()
end

function QuestKing:TitleButtonOnClick (mouse, down)
	local button = self.parent

	if ((button.mouseHandler) and (button.mouseHandler.TitleButtonOnClick)) then
		button.mouseHandler.TitleButtonOnClick(self, mouse, down)
		return
	end

	if (button.type == "collapser") then
		local headerName = button._headerName

		if QuestKingDBPerChar.collapsedHeaders[headerName] then
			QuestKingDBPerChar.collapsedHeaders[headerName] = nil
			wipe(QuestKing.newlyAddedQuests)
		else
			QuestKingDBPerChar.collapsedHeaders[headerName] = true
		end
		QuestKing:UpdateTracker()
	end
end

function QuestKing:ButtonOnEnter (motion)
	local button = self

	if ((button.mouseHandler) and (button.mouseHandler.TitleButtonOnEnter)) then
		button.mouseHandler.TitleButtonOnEnter(self, motion)
		return
	end
end

function QuestKing:ButtonOnLeave (motion)
	if (opt.tooltipScale) then
		local oldScale = GameTooltip.__QuestKingPreviousScale or 1
		GameTooltip.__QuestKingPreviousScale = nil
		GameTooltip:SetScale(oldScale)
	end
	GameTooltip:Hide()
end

function QuestKing:ButtonOnClick (mouse, down)
	local button = self

	if ((button.mouseHandler) and (button.mouseHandler.ButtonOnClick)) then
		button.mouseHandler.ButtonOnClick(self, mouse, down)
		return
	end
end
